#!/bin/sh

set -e

check_cert () {
    echo "Checking certificate on $(hostname -f):443"
    if ! openssl s_client -connect $(hostname -f):443 -CAfile /etc/ssl/certs/ca-certificates.crt < /dev/null > /tmp/openssl.out 2>&1 \
            || ! grep 'Verify return code: 0 (ok)' /tmp/openssl.out > /dev/null; then
        echo "Cert check failed"
        cat /tmp/openssl.out
        return 1
    fi
    echo "Cert check suceeded"
    return 0
}

appliance_type=$(cat /var/lib/lastline/product_codename)

new_cert_sha256='0bfb2dd67bf34c898fef8b9857030c4e5d0d6aae2665ba0ddd9339329b2b7825'
new_root_cert_sha256='1bee5bc7f7cf9d968dbf788c21bd625f07c8581eea0bdc2ce2a43560c9ed8bc6'
new_chain_sha256='bb9df52b6ec3430f7b262c03dcb1ba42c70d1c910d2fccd2a6d18ab8d7952b30'
cert_file='/etc/anonvpn/lastline_anonvpn.crt'
puppet_cert_file='/usr/share/appliance-config/modules/anonvpn/templates/lastline-ca-openvpn-02.crt.erb'
puppet_root_cert_file='/usr/share/appliance-config/modules/anonvpn/templates/lastline-ca.crt.erb'
root_cert_file='/usr/local/share/ca-certificates/lastline-root-ca-2.crt'

new_key_sha256='369cb700317b1e5b0374ea2f45ea7d2dd4cdc96c3166732a6b9371a5d8388da8'
key_file='/var/lib/lastline/9AFD8DA3.asc'
puppet_key_file='/usr/share/appliance-config/modules/appliance_update/files/9AFD8DA3.asc'

cert_needs_update=0
puppet_cert_needs_update=0
root_ca_missing=0

key_needs_update=0
puppet_key_needs_update=0

if ! [ -f $root_cert_file ]; then
    echo "*** Root cert ${root_cert_file} is missing"
    root_ca_missing=1
else
    echo "Root cert ${root_cert_file} is present"
fi

case "${appliance_type}" in
manager | analyst | pinbox | sensor)
    ;;
*)
    echo "*** Unsupported appliance type: ${appliance_type}"
    exit 1
    ;;
esac

if [ $appliance_type != "sensor" ]; then
    current_cert_sha256=$(sha256sum $cert_file | cut -d ' ' -f 1)
    current_puppet_cert_sha256=$(sha256sum $puppet_cert_file | cut -d ' ' -f 1)
    current_puppet_root_cert_sha256=$(sha256sum $puppet_root_cert_file | cut -d ' ' -f 1)

    if [ "${current_cert_sha256}" = "${new_chain_sha256}" ]; then
        echo "cert is up-to-date in ${cert_file}"
    else
        cert_needs_update=1
        echo "*** Old/unknown cert found in ${cert_file}"
    fi
    if [ "${current_puppet_cert_sha256}" = "${new_cert_sha256}" ]; then
        echo "cert is up-to-date in ${puppet_cert_file}"
    else
        puppet_cert_needs_update=1
        echo "*** Old/unknown cert found in ${puppet_cert_file}"
    fi
    if [ "${current_puppet_root_cert_sha256}" = "${new_root_cert_sha256}" ]; then
        echo "cert is up-to-date in ${puppet_root_cert_file}"
    else
        puppet_cert_needs_update=1
        echo "*** Old/unknown cert found in ${puppet_root_cert_file}"
    fi

    http_cert_needs_update=''
    for user_cert_file in /etc/ssl/certs/user.*.pem /etc/puppet/files/ssl-cert/user.*.pem; do
        if ! [ -f "$user_cert_file" ]; then
            echo "*** Cannot find /etc/ssl/certs/user.*.pem"
            exit 1
        fi

        if grep 'Serial Number: 15955383886438364209 ' $user_cert_file > /dev/null; then
            echo "*** Found old CA_signing04 in $user_cert_file"
            http_cert_needs_update="${http_cert_needs_update} $user_cert_file"
        else
            echo "Lastline-signed http certificate not in use for $user_cert_file"
        fi
    done
fi

current_key_sha256=$(sha256sum $key_file | cut -d ' ' -f 1)
current_puppet_key_sha256=$(sha256sum $puppet_key_file | cut -d ' ' -f 1)

if [ "${current_key_sha256}" = "${new_key_sha256}" ]; then
    echo "Key is up-to-date in ${key_file}"
else
    echo "*** Old/unknown key found in ${key_file}"
    key_needs_update=1
fi
if [ "${current_puppet_key_sha256}" = "${new_key_sha256}" ]; then
    echo "Key is up-to-date in ${puppet_key_file}"
else
    echo "*** Old/unknown key found in ${puppet_key_file}"
    puppet_key_needs_update=1
fi

if [ \
        $cert_needs_update = 0 -a \
        $puppet_cert_needs_update = 0 -a \
        "$http_cert_needs_update" = '' -a \
        $root_ca_missing = 0 -a \
        $key_needs_update = 0 -a \
        $puppet_key_needs_update = 0 \
    ]; then
    check_cert
    exit $?
fi

if [ "$1" != "update" ]; then
    echo "Pass 'update' as first argument to this script in order to perform an update"
    exit 1
fi

if [ $root_ca_missing = 1 ]; then
    echo "Creating ${root_cert_file}"
    cat > $root_cert_file <<EOF
-----BEGIN CERTIFICATE-----
MIIFkTCCA3mgAwIBAgIUesyKuC7PUfQWe8pL0wOUFa9bAKkwDQYJKoZIhvcNAQEL
BQAwUDELMAkGA1UEBhMCVVMxEzARBgNVBAgMCkNhbGlmb3JuaWExFzAVBgNVBAoM
Dkxhc3RsaW5lLCBJbmMuMRMwEQYDVQQDDApST09UIENBIFYyMB4XDTE5MTAyMTE5
MzEzOFoXDTM3MTAxNjE5MzEzOFowUDELMAkGA1UEBhMCVVMxEzARBgNVBAgMCkNh
bGlmb3JuaWExFzAVBgNVBAoMDkxhc3RsaW5lLCBJbmMuMRMwEQYDVQQDDApST09U
IENBIFYyMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEA3ka+zcfgB6Ik
3qT/FJVswqI6UZEvmEFiQoPp5HWiiL2YEhSRAfXH74xYS5tcDl1HqgWxK+cEZ2TK
phUCRUGmcLzm249TQ7m3P5rSnytn+zg0cq7nE9fWBR4/TlBJNoZdBcQhT+i6O6bE
TTm6uWWCKeBSqfk4XHb2wzQ6n1hqwaEUOewQL7VyTC9MIdYNH/Ef46+w2kCTVIwX
BSGXna02Q0t1jDzs/NfdlFnh7p4gwy+eeojvOLGPl1PV97gyNNlgmJ5WHPnpsK3s
euFnvVn5UAO9k56FJX6JpytJlBhwIJtgJS8C/Yuz0TySkRS+4yw6ZPCqtih98cSR
s+JMFJKK6pZOfdkMgg9XjE+SgNJNC/+BRFVIeKkUwmbmaYuzZRBdkioK59m54wjG
kTqtwCT++rrVUOaGiBYxzvcRzeMmSbTjSrPHgVAl2uDiElpEVzkY0XWpTAAMK01l
Vw6/L+0IRIBX+d93GOeu0RZNMj+j2Miw8CyzFwjNklh3I8fbQm+hK7AsBcp4yXjA
sMBAkBe+yVNkowvWHjynVNmjnVy4yfiKjXNxWwEgF0+OtjppM/O3y8JoUUYwVSNm
Nw73bD2zLiEFTVQUs0D9QlilSoOfTlIrxSmBAPPFTyEd2G+3jS2icgNilrKK3c3Y
4KMndHWckV4GHjDH2BgjrKDMtPbT1YMCAwEAAaNjMGEwHQYDVR0OBBYEFN12r9qK
EeA8FIBvVF+nusGBNE0bMB8GA1UdIwQYMBaAFN12r9qKEeA8FIBvVF+nusGBNE0b
MA8GA1UdEwEB/wQFMAMBAf8wDgYDVR0PAQH/BAQDAgGGMA0GCSqGSIb3DQEBCwUA
A4ICAQAzRDeBhvJCW+CiCDlL0gKgAd04KZQCIzIMMHsCig1fo57NFkWjIx80ecM0
/p3XEJYXzuz2/rDFsbYtZjvvFuM/fjDAcmh3OJ+HiOgrsN6bLR7HyT23VCpp6psd
a5SJcaBEmU0xvXa3wtP7R/TsZ3u+zSXVWtxSj4dl/cNu5dNPHJmHcF0MFtjOxJhC
UYhD/8yA/UYGXqYN89mYtaDCrC+8G2SVQ+42iCm4tHlBXp1qe2ARF/hSNGqJwkWq
gnp27ZZmwPL/aN1BN/0+78TKnZYX5gggv0rKwLVO5jDFQlj2boh/qIUSso5vdnb1
88nGyrChB78fXuBa3DFmeMYmLXQt3vcyC4RBRruvfgewwhsNOq/A/pyycjToxvkP
jmkeysX8Sr9ebwjE/U7VjlsRRbZHXRVWMaUY1cwINqR1ejgtFi2vN14wXXlOsdAU
cGUPVXje2FB3OqiOfvr/I/AubIOsyQinXgadV26ekCmySCNSQv3OO/yghgxBsyTq
0J5ex+GUK7rEwTq8bHPEzSXTmUhx0unt4SZHkCK3ql/Wi+KpzQhSB1yRX6SbPva6
ppXsyC3A4B3GQA2P869t4GNztmNh4tU9eK/XOhcWWMJGIFus2oHgU4glLaZ6Utb0
gaNY5bT+vJYbONhtux36jdhbr4XeFBuFvVMfP/wjL46jZG9mSg==
-----END CERTIFICATE-----
EOF
    echo "Updating ca-certificates"
    update-ca-certificates
fi

if [ $puppet_cert_needs_update = 1 ]; then
    echo "Updating cert in ${puppet_cert_file}"
    cat > ${puppet_cert_file} <<EOF
-----BEGIN CERTIFICATE-----
MIID8TCCAdmgAwIBAgIBFzANBgkqhkiG9w0BAQsFADBQMQswCQYDVQQGEwJVUzET
MBEGA1UECAwKQ2FsaWZvcm5pYTEXMBUGA1UECgwOTGFzdGxpbmUsIEluYy4xEzAR
BgNVBAMMClJPT1QgQ0EgVjIwHhcNMjIwNDE4MTc1MzM5WhcNMzYxMDIxMTc1MzM5
WjBaMQswCQYDVQQGEwJVUzETMBEGA1UECAwKQ2FsaWZvcm5pYTEXMBUGA1UECgwO
TGFzdGxpbmUsIEluYy4xHTAbBgNVBAMMFFNJR05JTkcgQ0EgT1BFTlZQTiAyMIGf
MA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCWOm+N9qbktYc8RkZuwQuJ0bybC8sH
HiMK1I3acMh2y1S2XImTU35ZgWW/Gt8znpf5EtmlCADhEnEwUjH1d80DGX/TEhZr
+bD2ouR2OyoVQKj29IzkK1rmKsYdn/A+yFWcuOgK8i6c+pjy1P9x0pFdlUZ90jy+
NRskKdiLCCk6RwIDAQABo1AwTjAdBgNVHQ4EFgQUlFwV9tpf2Q3zSOKtKmkPwCFC
A8kwHwYDVR0jBBgwFoAU3Xav2ooR4DwUgG9UX6e6wYE0TRswDAYDVR0TBAUwAwEB
/zANBgkqhkiG9w0BAQsFAAOCAgEASXH/BG/YjsF2/HKFisGJLojFi4aJiXcxWE5s
2Kfzi/CmESsV0XbF/9UzZJ8jfiw8BKIlF4sRt+DrqzAEGB+UxPX2XB77Wk8M4mXK
p2McvyDcKNXlRlSW1tDbsGXHP95u386snkdC6H4uU4AEY3M/r2Y5UCX9dWvQS6nP
+2ZnIw/jgDb9JRXKNnVIxUNouxt/FEJh7/10pUle5eRY6D5bXQm6o/19Jz8VC4Wz
o035i4ouTi3q1DMHnkMF5z4TElul2+NhKEXAfvEgUM189yA6i6jkid3m/C4vPNWf
HpaqjO+l7qSN7mR3OOWXMOPhC8Yw/LAIuzEuqlouRUSq9PbPvxIsFaYXaKkYnvv1
PLoa9R9Y1cWTk+O078ZS55Q5KtBnuL/nTOZ6ntZ0tXqAeuAiADnTQEeBASk40UEi
jtTVqZ9Q0k1FRSSaNfQ8Sw4aFawxYc+p/VnNwFZ8xAQxXzxiMbdzIEz77LbwmrTK
kvqDIdzEAgHkCPGt58xGaUPpAYARgZqH3pYYBweE1Q6RKrvp8vfqVtSaf0Hg+ko1
7WR5Vvxwm34AjPiam/d6K9/abhF+eM3vHWoNxxPi7L6qGy6PQ/OokECvgWsKSYDg
QABWfpknmslSQIbjhXza1pfSd+zuDTpCmdNt5OZPDuJNyWNUOZA84jWOeOmVeDMW
hh0sGss=
-----END CERTIFICATE-----
EOF
    echo "Updating cert in ${puppet_root_cert_file}"
    cp ${root_cert_file} ${puppet_root_cert_file}
fi

openvpn_crt1='/usr/share/appliance-config/modules/anonvpn/templates/openvpn-crt-anonvpn.lastline.com.erb'
openvpn_crt2='/usr/share/appliance-config/templates/analyst/vpn/openvpn-crt-anonvpn.lastline.com.erb'

if [ $cert_needs_update = 1 ]; then
    echo "Updating cert in ${cert_file}"
    if [ -e $openvpn_crt1 ]; then
        openpvn_crt_file=$openvpn_crt1
    elif [ -e $openvpn_crt2 ]; then
        openpvn_crt_file=$openvpn_crt2
    else
        echo "*** Cannot find openvpn-crt-anonvpn.lastline.com.erb"
        exit 1
    fi
    cat \
        $openpvn_crt_file \
        $puppet_cert_file \
        $puppet_root_cert_file \
        > $cert_file
    if grep 'anonvpn::mode.*lastline' /etc/appliance-config/local-settings.yaml > /dev/null && \
        [ "$(cat /var/lib/lastline/appliance_state)" = 'active' ]; then
        echo "Restarting anonvpn client"
        ifdown llanonvpn0
        ifup llanonvpn0
    else
        echo "Appliance not using anonvpn, or standby manager, not restarting the client"
    fi
fi

if [ "$http_cert_needs_update" != '' ]; then
    for user_cert_file in $http_cert_needs_update; do
        user_cert_file_backup="${user_cert_file}.$(date +%s)"
        echo "Saving ${user_cert_file} to ${user_cert_file_backup}"
        cp $user_cert_file $user_cert_file_backup
        sed '/-----END CERTIFICATE-----/q' $user_cert_file_backup > $user_cert_file
        cat >> $user_cert_file <<EOF
-----BEGIN CERTIFICATE-----
MIIEbTCCAlWgAwIBAgIBGDANBgkqhkiG9w0BAQsFADBQMQswCQYDVQQGEwJVUzET
MBEGA1UECAwKQ2FsaWZvcm5pYTEXMBUGA1UECgwOTGFzdGxpbmUsIEluYy4xEzAR
BgNVBAMMClJPT1QgQ0EgVjIwHhcNMjIwNDE4MTc1NjIyWhcNMzYxMDIxMTc1NjIy
WjBSMQswCQYDVQQGEwJVUzETMBEGA1UECAwKQ2FsaWZvcm5pYTEXMBUGA1UECgwO
TGFzdGxpbmUsIEluYy4xFTATBgNVBAMMDENBX3NpZ25pbmcwNDCCASIwDQYJKoZI
hvcNAQEBBQADggEPADCCAQoCggEBAMysCv/1ya+QLVqvqP4mowEGtmzHM6PHfgEB
3CotinxFFlCe01hazaHj7eG2nbWW8sW7X8MN0W46GHYGNYLpX1MbxUebWPpNSJ4P
dpZpye2BwZoN9lXRMSeNLQvpLZZtbvJ1Hnk/Om/qg0614KsAHPR1HMdfUUP57oOX
68Cx5F/2Oo/1GMcYgCmCGeSiC/CCCW0NsdYHFOV+r+kwCRJDjBkdl7DzTZNwMFHg
X/TGyEtR6vNyLGWs6Snvi9L4gn7zGxMcm8X4EevPpfZoQH/aqcAN2KF3e6M9JICL
X2D50jfBmNop1lyNYSEIpKrxV2u929d2amlvr4ZSp63hq3+LuS8CAwEAAaNQME4w
HQYDVR0OBBYEFNHnb4d1DuMQ9HLUiwEsd8BCPo1rMB8GA1UdIwQYMBaAFN12r9qK
EeA8FIBvVF+nusGBNE0bMAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQELBQADggIB
AM0dl4yjZS7DCndKQoEKC+TmvMrwQuXxnglLMkT/gLssQrZpMa/OkNT9YmaQ6QJh
iFnkf27ihixJBWe+0XwN0S3l5Akg1yOja2Xe9v3eaQwXd0WLykTHPnO+cmsu/BYc
8QbConcjWGlkgBgtZhwnHbDd/zxIi+M3QB/hFaQv4SVTDnuE7ivzn7fYknNGPaEv
fuznWVHQmcbeSl5hBg+F+7cbFIL4BnGCCB3z8iRaypcnu5DE5Pa0zsxzzP+5Z3o6
+lrYMlUpJYMIv3QE3j5N40uM+53rGkHCfVtHzyRXWozOpZQUPyGGelOWGgY2bV24
nQO9SVQGLIYv4sD2OJzBzsTY3nbXLYywibTXcy9y9u7LFN1hMkClzrmeoDJprVUn
FWbOaqOGOfcykQsjao4K7QdEAm5cdx9fXh7Xe6hFpPm5baPMdSBfg1FNuqQuj/5P
UaiO7JIt9RMs2mZYq76GsxjOb3MJenmVzCIIaSbiLZ4HMNCy1Zl0Ob6+ZPsSiQhx
5z5YdLvtfKctYeeHjEj4Psxp58Oj+x4N8qUot9VBO7anmnFknlnJ1ltckrtMrmCt
cANPwHnIuEZYCS43BhsuqFlk4KELCpfstd4XlpcMcJpsc0e31nC+4kssgOtIgymf
m/CKLkrZAgpFbcWCRn/jAYssEF8VjiT2xpJOsti+txDP
-----END CERTIFICATE-----
EOF
    done
    echo "Reloading nginx configuration"
    nginx -s reload
    echo "Updating ca-certificates"
    update-ca-certificates
    sleep 3
fi

if [ $puppet_key_needs_update = 1 ]; then
    echo "Updating key in ${puppet_key_file}"
    cat > ${puppet_key_file} <<EOF
-----BEGIN PGP PUBLIC KEY BLOCK-----

mQINBFrU+J8BEAC7ujh09AoL0lUtsnhAj9OCQxZqbGJWVkZs0iJQ637YY/Yh6mMy
MqK3howYdksSLJSEp12gyo96GxLb1A1BU0GDID5UU2Jh/lRZrsw/75cqd+icZhCv
ajAkwM3gYXqeIKmA553mZNP4FZnNWdXFm8DRgdrx1dc4vYKaVIyouSQAeKPpGcBh
LZcEvT1cOmk5i8wHl82plC3k01/TkAe2rD68aReWeRZPnHH0mXw5xko/8ZlcWjAc
+BJrjKNZZ8sG4luKlKjl0iSrMvaH57RBYNB/ypUbjOEfxLQALOLX/nuLRiOzG+B8
KRqR0SY2a602Ij2ikh51aOotrsopBerxiyVTRaLZVmIpN2Q5hO3RYJSmrVmk4b5I
DQYo+5WP/KnXHJmtnvGXLdzEzffjkqbA61zZIuoI2At87xQQuaYxyiYlzm59Nnl0
csXfv56WIpZtRaSE5wSVquKI+HF0lWhig0C2LvEm1L+X87p9boAZAsSZs+XtJrcl
LN58TJaC93vYwrdYhs2KlrKLWuajV4HThEWdm73VMsN5s95fv1ytGAssgkRG2vPK
hyiSnyZ/+JsG6vlW6C2gmgmL6paHgSI6xe6PvMPQMjQKHHt0KqucWkTbk1igltMS
g12JoNCx8mOxLAMtZI5osF1ARV0fvfU+wBwobvtpz2ejwHExmt+Fr0DntwARAQAB
tG9hcHBsaWFuY2UtbWdtdEBsYXN0bGluZS5jb20gKEZvciBzaWduaW5nIG9mIGNv
bW1hbmRzIGFuZCBtZXNzYWdlcyB0byBhcHBsaWFuY2VzLikgPGFwcGxpYW5jZS1t
Z210QGxhc3RsaW5lLmNvbT6JAlUEEwEKAD8CGwMGCwkIBwMCBhUIAgkKCwQWAgMB
Ah4BAheAFiEE9X34mVl8dXtVG7UP8RyQ5Jr9jaMFAmI83NwFCSV7HD0ACgkQ8RyQ
5Jr9jaObpg//TmRcHdwotnelXxS8YNXhOzweylX/pE3gTSuwq7aMDo3B4A/mM+y4
vlG0Ze1t6dFZzOO78ZkbZnumWVhGOk4z5bsbVdbJXL4IjYpS4pF3NpH415eNtJN5
Oj8jU4owJ7M8oUO0T1CBg2ibFpDcembCVARAdQAWAvXVR1z/OFeb/8+CVEg5uEZz
tJTp6rw9lKhic2yLRWLPQ2wezlxbMHUbC1u/LMpFS1Ukvb+T5t9LPVZ7aI5ZtPjl
1qbQq1ns42O8hIRR6Py60Yyq+abpZ/znkQt23Sv+ioB0pSJYPyO6SNYHz4wgvZ7W
PmsKeAUp0xcHxh0V+RuGu/Vc593d1/n0lxmpOC4/6u4lNl9k1T2sdhMOdhovUksG
QjTH17pi8xED5csT3Qe9v+DIFjUXeI7XRjWQdc3jDb893dRZ9XYqmVB246EMY6Nx
IIK6nHUp0eWp8uAk1J+DifzVzmnuzlJERjc2XVevk0kTnFHVN2OdqOSCXY4l0m5v
3lVVDM5ERKSIhdw1ZKWog7lkUjwt4zAL0GujH838bkjUBizsYG1HpUPY7Ui3DUv6
EuR5u7DoR1qv65H+3dQPVH53BvEZC8oK19LjtAVFO5zm/Ya/uaaFCB1imIuwuZK6
TSiPZsYz5R1Plk08RhlXMOyqGbt3aqVhu1Ljnudt8AJZvKskZgbT95k=
=OCjD
-----END PGP PUBLIC KEY BLOCK-----
EOF
fi

if [ $key_needs_update = 1 ]; then
    echo "Updating key in ${key_file}"
    cp $puppet_key_file $key_file
    echo "Scheduling restart of appliance-update daemon"
    touch /tmp/appliance_update.restart
fi

if [ $appliance_type != 'sensor' ]; then
    check_cert
fi
exit $?
